@@ -44,7 +44,7 @@ use Glib::Object::Subclass Gtk3::Window::,
     ),
   ];
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.1.0';
 my $EMPTY    = q{};
 my $HEXREGEX = qr{^(.*)           # start of message
                   \b0x[[:xdigit:]]+\b # hex (e.g. address)
@@ -147,9 +147,10 @@ sub add_metadata_dialog {
     $entryd->signal_connect( 'insert-text' => \&insert_text_handler );
     $entryd->signal_connect(
         'focus-out-event' => sub {
-            my $text = Gscan2pdf::Document::text_to_date( $entryd->get_text );
-            if ( defined $text ) {
-                $entryd->set_text( sprintf '%04d-%02d-%02d', $text );
+            my $text = $entryd->get_text;
+            if ( defined $text and $text ne $EMPTY ) {
+                $entryd->set_text( sprintf '%04d-%02d-%02d',
+                    Gscan2pdf::Document::text_to_date($text) );
             }
             return FALSE;
         }
