@@ -13,6 +13,7 @@ use Gscan2pdf::Dialog;
 use Gscan2pdf::Scanner::Options;
 use Gscan2pdf::Scanner::Profile;
 use Gscan2pdf::Translation '__';    # easier to extract strings with xgettext
+use List::MoreUtils qw(first_index);
 use Readonly;
 Readonly my $BORDER_WIDTH => 6;
 
@@ -267,7 +268,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.1.0';
 
 my ( $d_sane, $logger );
 my $tolerance             = 1;
@@ -1212,6 +1213,8 @@ sub get_paper_by_geometry {
 
 sub update_options {
     my ( $self, $new_options ) = @_;
+    $logger->debug( 'Sane->get_option_descriptor returned: ',
+        Dumper($new_options) );
 
     my $loops = $self->get('num-reloads');
     $self->{num_reloads} = ++$loops;    # num-reloads is read-only
@@ -1234,34 +1237,7 @@ sub update_options {
     # set.
     my $current_scan_options = dclone( $self->{current_scan_options} );
 
-    # Loop through the cloned profile, and remove those options that have
-    # correct values in the reload
-    my $num = $current_scan_options->num_backend_options;
-    my $i   = 0;
-    while ( $i < $num ) {
-        my ( $name, $val ) =
-          $current_scan_options->get_backend_option_by_index($i);
-        my $opt = $new_options->by_name($name);
-        if ( not defined $val
-            or Gscan2pdf::Scanner::Options::within_tolerance( $opt, $val ) )
-        {
-            $logger->info(
-                "No need to update option '$name': already within tolerance.");
-            $current_scan_options->remove_backend_option_by_name($name);
-            --$num;
-        }
-        else {
-            $logger->info(
-"Setting option '$name' to '$val' (again): '$opt->{val}' out of tolerance."
-            );
-            ++$i;
-        }
-    }
-
     # walk the widget tree and update them from the hash
-    $logger->debug( 'Sane->get_option_descriptor returned: ',
-        Dumper($new_options) );
-
     my $num_dev_options = $new_options->num_options;
     my $options         = $self->get('available-scan-options');
     for ( 1 .. $num_dev_options - 1 ) {
@@ -2204,6 +2180,25 @@ sub _set_option_profile {
             return;
         }
 
+        # Don't try to set invalid option
+        if ( $opt->{constraint} and ref( $opt->{constraint} ) eq 'ARRAY' ) {
+            my $index = first_index { $_ eq $val } @{ $opt->{constraint} };
+            if ( $index == $NO_INDEX ) {
+                $logger->warn(
+                    "Ignoring invalid argument '$val' for option '$name'.");
+                $self->_set_option_profile( $profile, $next );
+                return;
+            }
+        }
+
+        # Ignore option if value already within tolerance
+        if ( Gscan2pdf::Scanner::Options::within_tolerance( $opt, $val ) ) {
+            $logger->info(
+                "No need to set option '$name': already within tolerance.");
+            $self->_set_option_profile( $profile, $next );
+            return;
+        }
+
         $logger->debug(
             "Setting option '$name'"
               . (
@@ -2218,7 +2213,7 @@ sub _set_option_profile {
                 my ( $widget, $optname, $optval, $uuid ) = @_;
 
                 # With multiple reloads, this can get called several times,
-                # so only react to to signal from the correct profile
+                # so only react to signal from the correct profile
                 if ( defined $uuid and $uuid eq $profile->{uuid} ) {
                     $self->signal_handler_disconnect($signal);
                     $self->_set_option_profile( $profile, $next );
@@ -2270,13 +2265,9 @@ sub update_widget_value {
             }
             when ( $widget->isa('Gtk3::ComboBox') ) {
                 if ( $opt->{constraint}[ $widget->get_active ] ne $val ) {
-                    my $index;
-                    for ( 0 .. $#{ $opt->{constraint} } ) {
-                        if ( $opt->{constraint}[$_] eq $val ) {
-                            $index = $_;
-                        }
-                    }
-                    if ( defined $index ) { $widget->set_active($index) }
+                    my $index =
+                      first_index { $_ eq $val } @{ $opt->{constraint} };
+                    if ( $index > $NO_INDEX ) { $widget->set_active($index) }
                 }
             }
             when ( $widget->isa('Gtk3::Entry') ) {
