@@ -41,7 +41,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, properties => [
     ),
 ];
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.1.0';
 
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
@@ -87,6 +87,7 @@ sub get_devices {
 
             # Set up ProgressBar
             $pbar = Gtk3::ProgressBar->new;
+            $pbar->set_show_text(TRUE);
             $pbar->set_pulse_step( $self->get('progress-pulse-step') );
             $pbar->set_text( __('Fetching list of devices') );
             $hboxd->pack_start( $pbar, TRUE, TRUE, 0 );
@@ -482,12 +483,9 @@ sub scan {
                 my $current = $self->get('current-scan-options');
                 my $signal;
                 $signal = $self->signal_connect(
-                    'finished-process' => sub {
-                        my ( $widget, $data ) = @_;
-                        if ( $data eq 'find_scan_options' ) {
-                            $self->signal_handler_disconnect($signal);
-                            $self->set_current_scan_options($current);
-                        }
+                    'reloaded-scan-options' => sub {
+                        $self->signal_handler_disconnect($signal);
+                        $self->set_current_scan_options($current);
                     }
                 );
                 $self->scan_options( $self->get('device') );
