@@ -10,9 +10,28 @@ use Gtk3;
 use List::Util qw(min);
 use Carp;
 use Readonly;
-Readonly my $HALF => 0.5;
+Readonly my $HALF          => 0.5;
+Readonly my $CURSOR_PIXELS => 5;
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.1.0';
+
+my %cursorhash = (
+    left => {
+        top    => 'top-left-corner',
+        mid    => 'left-side',
+        bottom => 'bottom-left-corner',
+    },
+    mid => {
+        top    => 'top-side',
+        mid    => 'crosshair',
+        bottom => 'bottom-side',
+    },
+    right => {
+        top    => 'top-right-corner',
+        mid    => 'right-side',
+        bottom => 'bottom-right-corner',
+    },
+);
 
 # Note: in a BEGIN block to ensure that the registration is complete
 #       by the time the use Subclass goes to look for it.
@@ -72,6 +91,14 @@ use Glib::Object::Subclass Gtk3::DrawingArea::, signals => {
         'Gdk::Rectangle hash of selected region',    # blurb
         [qw/readable writable/]                      # flags
     ),
+    Glib::ParamSpec->boolean(
+        'zoom-to-fit',                               # name
+        'Zoom to fit',                               # nickname
+        'Whether the zoom factor is automatically calculated to fit the window'
+        ,                                            # blurb
+        TRUE,                                        # default
+        [qw/readable writable/]                      # flags
+    ),
   ];
 
 sub INIT_INSTANCE {
@@ -81,7 +108,9 @@ sub INIT_INSTANCE {
     $self->signal_connect( 'button-release-event' => \&_button_released );
     $self->signal_connect( 'motion-notify-event'  => \&_motion );
     $self->signal_connect( 'scroll-event'         => \&_scroll );
+    $self->signal_connect( configure_event        => \&_configure_event );
     $self->set_app_paintable(TRUE);
+
     if (
         $Glib::Object::Introspection::VERSION <
         0.043    ## no critic (ProhibitMagicNumbers)
@@ -110,6 +139,7 @@ sub INIT_INSTANCE {
         );
     }
     $self->set_tool('dragger');
+    $self->set_redraw_on_allocate(FALSE);
     return $self;
 }
 
@@ -124,9 +154,7 @@ sub SET_PROPERTY {
         given ($name) {
             when ('pixbuf') {
                 $self->{$name} = $newval;
-                if ( not defined $newval ) {
-                    $invalidate = TRUE;
-                }
+                $invalidate = TRUE;
             }
             when ('zoom') {
                 $self->{$name} = $newval;
@@ -172,10 +200,7 @@ sub SET_PROPERTY {
             }
         }
         if ($invalidate) {
-            my $win = $self->get_window();
-            if ( defined $win ) {
-                $win->invalidate_rect( $self->get_allocation, FALSE );
-            }
+            $self->queue_draw();
         }
     }
     return;
@@ -184,10 +209,8 @@ sub SET_PROPERTY {
 sub set_pixbuf {
     my ( $self, $pixbuf, $zoom_to_fit ) = @_;
     $self->set( 'pixbuf', $pixbuf );
-    if ($zoom_to_fit) {
-        $self->zoom_to_fit;
-    }
-    else {
+    $self->set_zoom_to_fit($zoom_to_fit);
+    if ( not $zoom_to_fit ) {
         $self->set_offset( 0, 0 );
     }
     return;
@@ -215,17 +238,20 @@ sub _button_pressed {
 
     $self->{drag_start} = { x => $event->x, y => $event->y };
     $self->{dragging} = TRUE;
+    $self->update_cursor( $event->x, $event->y );
     return TRUE;
 }
 
 sub _button_released {
     my ( $self, $event ) = @_;
     $self->{dragging} = FALSE;
+    $self->update_cursor( $event->x, $event->y );
     return;
 }
 
 sub _motion {
     my ( $self, $event ) = @_;
+    $self->update_cursor( $event->x, $event->y );
     if ( not $self->{dragging} ) { return FALSE }
 
     if ( $self->get_tool eq 'dragger' ) {
@@ -240,14 +266,52 @@ sub _motion {
         $self->set_offset( $offset_x, $offset_y );
     }
     elsif ( $self->get_tool eq 'selector' ) {
-
-        # calculate the rectangle, give it the right orientation
-        my $x = int( min( $self->{drag_start}{x}, $event->x ) );
-        my $y = int( min( $self->{drag_start}{y}, $event->y ) );
-        my $w = int( abs( $self->{drag_start}{x} - $event->x ) );
-        my $h = int( abs( $self->{drag_start}{y} - $event->y ) );
-        ( $x, $y ) = $self->_to_image_coords( $x, $y );
-        ( $w, $h ) = $self->_to_image_distance( $w, $h );
+        my ( $x, $y, $x2, $y2, $x_old, $y_old, $x2_old, $y2_old );
+        if ( $self->{h_edge} eq 'left' ) {
+            $x = $event->x;
+        }
+        elsif ( $self->{h_edge} eq 'right' ) {
+            $x2 = $event->x;
+        }
+        if ( $self->{v_edge} eq 'top' ) {
+            $y = $event->y;
+        }
+        elsif ( $self->{v_edge} eq 'bottom' ) {
+            $y2 = $event->y;
+        }
+        if ( $self->{h_edge} eq 'mid' and $self->{v_edge} eq 'mid' ) {
+            $x  = $self->{drag_start}{x};
+            $x2 = $event->x;
+            $y  = $self->{drag_start}{y};
+            $y2 = $event->y;
+        }
+        else {
+            my $selection = $self->get_selection;
+            if ( not defined $x or not defined $y ) {
+                ( $x_old, $y_old ) =
+                  $self->_to_widget_coords( $selection->{x}, $selection->{y} );
+            }
+            if ( not defined $x2 or not defined $y2 ) {
+                ( $x2_old, $y2_old ) = $self->_to_widget_coords(
+                    $selection->{x} + $selection->{width},
+                    $selection->{y} + $selection->{height}
+                );
+            }
+            if ( not defined $x ) {
+                $x = $x_old;
+            }
+            if ( not defined $x2 ) {
+                $x2 = $x2_old;
+            }
+            if ( not defined $y ) {
+                $y = $y_old;
+            }
+            if ( not defined $y2 ) {
+                $y2 = $y2_old;
+            }
+        }
+        my ( $w, $h ) = $self->_to_image_distance( abs $x2 - $x, abs $y2 - $y );
+        ( $x, $y ) = $self->_to_image_coords( min( $x, $x2 ), min( $y, $y2 ) );
         $self->set_selection( { x => $x, y => $y, width => $w, height => $h } );
     }
     return;
@@ -259,6 +323,7 @@ sub _scroll {
     my ( $center_x, $center_y ) =
       $self->_to_image_coords( $event->x, $event->y );
     my $zoom;
+    $self->set_zoom_to_fit(FALSE);
     if ( $event->direction eq 'up' ) {
         $zoom = $self->get_zoom * 2;
     }
@@ -314,7 +379,24 @@ sub _draw {
     return TRUE;
 }
 
+sub _configure_event {
+    my ( $self, $event ) = @_;
+    if ( $self->get_zoom_to_fit ) {
+        $self->_calculate_zoom_to_fit;
+    }
+    return;
+}
+
+# setting the zoom via the public API disables zoom-to-fit
+
 sub set_zoom {
+    my ( $self, $zoom ) = @_;
+    $self->set_zoom_to_fit(FALSE);
+    $self->_set_zoom_no_center($zoom);
+    return;
+}
+
+sub _set_zoom {
     my ( $self, $zoom ) = @_;
     $self->set( 'zoom', $zoom );
     return;
@@ -354,7 +436,7 @@ sub _set_zoom_with_center {
     my $allocation = $self->get_allocation;
     my $offset_x   = $allocation->{width} / 2 / $zoom - $center_x;
     my $offset_y   = $allocation->{height} / 2 / $zoom - $center_y;
-    $self->set_zoom($zoom);
+    $self->_set_zoom($zoom);
     $self->set_offset( $offset_x, $offset_y );
     return;
 }
@@ -370,7 +452,15 @@ sub _set_zoom_no_center {
     return;
 }
 
-sub zoom_to_fit {
+sub set_zoom_to_fit {
+    my ( $self, $zoom_to_fit ) = @_;
+    $self->set( 'zoom-to-fit', $zoom_to_fit );
+    if ( not $zoom_to_fit ) { return }
+    $self->_calculate_zoom_to_fit;
+    return;
+}
+
+sub _calculate_zoom_to_fit {
     my ($self) = @_;
     my $pixbuf_size = $self->get_pixbuf_size;
     if ( not defined $pixbuf_size ) { return }
@@ -385,18 +475,31 @@ sub zoom_to_fit {
     return;
 }
 
+sub get_zoom_to_fit {
+    my ($self) = @_;
+    return $self->get('zoom-to-fit');
+}
+
 sub zoom_in {
     my ($self) = @_;
+    $self->set_zoom_to_fit(FALSE);
     $self->_set_zoom_no_center( $self->get_zoom * 2 );
     return;
 }
 
 sub zoom_out {
     my ($self) = @_;
+    $self->set_zoom_to_fit(FALSE);
     $self->_set_zoom_no_center( $self->get_zoom / 2 );
     return;
 }
 
+sub zoom_to_fit {
+    my ($self) = @_;
+    $self->set_zoom_to_fit(TRUE);
+    return;
+}
+
 sub _clamp_direction {
     my ( $offset, $allocation, $pixbuf_size ) = @_;
 
@@ -499,4 +602,62 @@ sub get_selection {
     return $self->get('selection');
 }
 
+sub update_cursor {
+    my ( $self, $x, $y ) = @_;
+    my $pixbuf_size = $self->get_pixbuf_size;
+    if ( not defined $pixbuf_size ) { return }
+    my $win     = $self->get_window;
+    my $display = Gtk3::Gdk::Display::get_default;
+    my $tool    = $self->get_tool;
+    my $cursor;
+
+    if ( $tool eq 'dragger' ) {
+        ( $x, $y ) = $self->_to_image_coords( $x, $y );
+        if (    $x > 0
+            and $x < $pixbuf_size->{width}
+            and $y > 0
+            and $y < $pixbuf_size->{height} )
+        {
+            if ( $self->{dragging} ) {
+                $cursor = Gtk3::Gdk::Cursor->new('hand2');
+            }
+            else {
+                $cursor = Gtk3::Gdk::Cursor->new('hand1');
+            }
+        }
+    }
+    elsif ( $tool eq 'selector' ) {
+        if ( $self->{dragging} ) { return }
+        my $selection = $self->get_selection;
+        my ( $sx1, $sy1 ) =
+          $self->_to_widget_coords( $selection->{x}, $selection->{y} );
+        my ( $sx2, $sy2 ) = $self->_to_widget_coords(
+            $selection->{x} + $selection->{width},
+            $selection->{y} + $selection->{height}
+        );
+        ( $self->{h_edge}, $self->{v_edge} ) = qw( mid mid );
+        if ( _between( $x, $sx1 - $CURSOR_PIXELS, $sx1 + $CURSOR_PIXELS ) ) {
+            $self->{h_edge} = 'left';
+        }
+        elsif ( _between( $x, $sx2 - $CURSOR_PIXELS, $sx2 + $CURSOR_PIXELS ) ) {
+            $self->{h_edge} = 'right';
+        }
+        if ( _between( $y, $sy1 - $CURSOR_PIXELS, $sy1 + $CURSOR_PIXELS ) ) {
+            $self->{v_edge} = 'top';
+        }
+        elsif ( _between( $y, $sy2 - $CURSOR_PIXELS, $sy2 + $CURSOR_PIXELS ) ) {
+            $self->{v_edge} = 'bottom';
+        }
+        $cursor = Gtk3::Gdk::Cursor->new(
+            $cursorhash{ $self->{h_edge} }{ $self->{v_edge} } );
+    }
+    $win->set_cursor($cursor);
+    return;
+}
+
+sub _between {
+    my ( $value, $lower, $upper ) = @_;
+    return ( $value > $lower and $value < $upper );
+}
+
 1;
